from osdan import filters as fl

names = {'DISPATCHLOAD': 'PUBLIC_DVD_DISPATCHLOAD',
         'DUDETAILSUMMARY': 'PUBLIC_DVD_DUDETAILSUMMARY',
         'DISPATCHCONSTRAINT': 'PUBLIC_DVD_DISPATCHCONSTRAINT',
         'GENCONDATA': 'PUBLIC_DVD_GENCONDATA',
         'DISPATCH_UNIT_SCADA': 'PUBLIC_DVD_DISPATCH_UNIT_SCADA',
         'DISPATCHPRICE': 'PUBLIC_DVD_DISPATCHPRICE',
         'SPDREGIONCONSTRAINT': 'PUBLIC_DVD_SPDREGIONCONSTRAINT',
         'SPDCONNECTIONPOINTCONSTRAINT': 'PUBLIC_DVD_SPDCONNECTIONPOINTCONSTRAINT',
         'SPDINTERCONNECTORCONSTRAINT': 'PUBLIC_DVD_SPDINTERCONNECTORCONSTRAINT',
         'BIDPEROFFER_D': 'PUBLIC_DVD_BIDPEROFFER_D',
         'DISPATCHINTERCONNECTORRES': 'PUBLIC_DVD_DISPATCHINTERCONNECTORRES',
         'BIDDAYOFFER_D': 'PUBLIC_DVD_BIDDAYOFFER_D',
         'DISPATCHREGIONSUM': 'PUBLIC_DVD_DISPATCHREGIONSUM',
         'FCAS_4_SECOND': 'FCAS',
         'ELEMENTS_FCAS_4_SECOND': 'Elements_FCAS.csv',
         'VARIABLES_FCAS_4_SECOND': '820-0079 csv.csv',
         'MASTER_REGISTRATION_LIST': 'NEM Registration and Exemption List'}

parent_tables = ['DUDETAILSUMMARY', 'INTERCONNECTORCONSTRAINT', 'INTERCONNECTOR', 'LOSSMODEL', 'MNSP_INTERCONNECTOR',
                 'MNSP_PEROFFER', 'DISPATCHCONSTRAINT', 'BIDPEROFFER_D', 'DISPATCHLOAD', 'GENCONDATA',
                 'SPDCONNECTIONPOINTCONSTRAINT', 'SPDINTERCONNECTORCONSTRAINT', 'DISPATCHINTERCONNECTORRES',
                 'DISPATCHPRICE', 'SPDREGIONCONSTRAINT', 'BIDDAYOFFER_D', 'MNSP_DAYOFFER', 'LOSSFACTORMODEL',
                 'DISPATCHREGIONSUM']

child_parent_map = {'connection_point_constraints': ['SPDCONNECTIONPOINTCONSTRAINT'],
                    'constraint_data': ['GENCONDATA', 'DISPATCHCONSTRAINT'],
                    'interconnector_constraints': ['SPDINTERCONNECTORCONSTRAINT'],
                    'capacity_bids': ['BIDPEROFFER_D'], # Dispatch load should get merged here.
                    'interconnectors': ['INTERCONNECTOR', 'INTERCONNECTORCONSTRAINT'],
                    'market_interconnectors': ['MNSP_INTERCONNECTOR'],
                    'generator_information': ['DUDETAILSUMMARY'],
                    'region_constraints': ['SPDREGIONCONSTRAINT'],
                    'price_bids': ['BIDDAYOFFER_D'],
                    'market_interconnector_price_bids': ['MNSP_DAYOFFER'],
                    'market_interconnector_capacity_bids': ['MNSP_PEROFFER'],
                    'initial_conditions': ['DISPATCHLOAD'],
                    'interconnector_segments': ['LOSSMODEL', 'DISPATCHINTERCONNECTORRES'],
                    'interconnector_dynamic_loss_coefficients': ['LOSSFACTORMODEL'],
                    'demand': ['DISPATCHREGIONSUM']}

parent_merge_cols = {'connection_point_constraints': None,
                     'constraint_data': ['GENCONID', 'EFFECTIVEDATE', 'VERSIONNO'],
                     'interconnector_constraints': None,
                     'capacity_bids': ['DUID'],
                     'interconnectors': ['INTERCONNECTORID'],
                     'market_interconnectors': None,
                     'generator_information': None,
                     'region_constraints': None,
                     'price_bids': None,
                     'market_interconnector_price_bids': None,
                     'market_interconnector_capacity_bids': None,
                     'initial_conditions': None,
                     'interconnector_segments': ['INTERCONNECTORID'],
                     'interconnector_dynamic_loss_coefficients': None,
                     'demand': None}

required_cols = {'SPDCONNECTIONPOINTCONSTRAINT': None,
                 'GENCONDATA': None,
                 'SPDINTERCONNECTORCONSTRAINT': None,
                 'BIDPEROFFER_D': None,
                 'DISPATCHINTERCONNECTORRES': ('INTERCONNECTORID', 'DISPATCHINTERVAL'),
                 'INTERCONNECTOR': ('INTERCONNECTORID', 'REGIONFROM', 'REGIONTO'),
                 'INTERCONNECTORCONSTRAINT': ('INTERCONNECTORID', 'MAXMWIN', 'MAXMWOUT', 'LOSSCONSTANT',
                                              'LOSSFLOWCOEFFICIENT', 'FROMREGIONLOSSSHARE', 'ICTYPE'),
                 'MNSP_INTERCONNECTOR': ('INTERCONNECTORID', 'LINKID', 'FROMREGION', 'TOREGION', 'MAXCAPACITY',
                                         'FROM_REGION_TLF', 'TO_REGION_TLF', 'LHSFACTOR'),
                 'DISPATCHPRICE': None,
                 'DUDETAILSUMMARY': ('DUID', 'END_DATE', 'DISPATCHTYPE', 'CONNECTIONPOINTID', 'REGIONID', 'LASTCHANGED',
                                     'TRANSMISSIONLOSSFACTOR', 'DISTRIBUTIONLOSSFACTOR'),
                 'DISPATCHCONSTRAINT': ('GENCONID', 'RHS', 'VERSIONNO', 'EFFECTIVEDATE'),
                 'SPDREGIONCONSTRAINT': None,
                 'BIDDAYOFFER_D': None,
                 'MNSP_DAYOFFER': ('LINKID', 'PRICEBAND1', 'PRICEBAND2', 'PRICEBAND3', 'PRICEBAND4', 'PRICEBAND5',
                                   'PRICEBAND6', 'PRICEBAND7', 'PRICEBAND8', 'PRICEBAND9', 'PRICEBAND10'),
                 'MNSP_PEROFFER': ('LINKID', 'BANDAVAIL1', 'BANDAVAIL2', 'BANDAVAIL3', 'BANDAVAIL4', 'BANDAVAIL5',
                                   'BANDAVAIL6', 'BANDAVAIL7', 'BANDAVAIL8', 'BANDAVAIL9', 'BANDAVAIL10', 'MAXAVAIL',
                                   'RAMPUPRATE'),
                 'DISPATCHLOAD': None,
                 'LOSSMODEL': None,
                 'LOSSFACTORMODEL': None,
                 'DISPATCHREGIONSUM': None}

wrapper_map = {'SPDCONNECTIONPOINTCONSTRAINT': None,
               'GENCONDATA': None,
               'SPDINTERCONNECTORCONSTRAINT': None,
               'BIDPEROFFER_D': fl.filter_on_interval_datetime,
               'DISPATCHINTERCONNECTORRES': fl.filter_on_settlementdate,
               'INTERCONNECTOR': fl.filter_on_last_changed,
               'INTERCONNECTORCONSTRAINT': fl.filter_on_effective_date,
               'MNSP_INTERCONNECTOR': fl.filter_on_effective_date,
               'DISPATCHPRICE': fl.filter_on_settlementdate,
               'DUDETAILSUMMARY': fl.filter_on_start_and_end_date,
               'DISPATCHCONSTRAINT': fl.filter_on_settlementdate,
               'SPDREGIONCONSTRAINT': None,
               'BIDDAYOFFER_D': fl.filter_on_settlementdate,
               'MNSP_DAYOFFER': fl.filter_on_settlementdate,
               'MNSP_PEROFFER': fl.filter_on_date_and_peroid,
               'DISPATCHLOAD': fl.filter_on_settlementdate,
               'LOSSMODEL': fl.filter_on_effective_date,
               'LOSSFACTORMODEL': fl.filter_on_effective_date,
               'DISPATCHREGIONSUM': fl.filter_on_settlementdate}

aemo_data_suffix = 'PUBLIC_DVD_'



import os
import pandas as pd
from osdan import data_fetch_methods
from osdan import defaults
from osdan import query_wrapers
from datetime import datetime, timedelta
from joblib import Parallel, delayed
from nemlite import nemlite_defaults


def actual_inputs_replicator(start_time, end_time, raw_aemo_data_folder, filtered_data_folder, run_pre_filter=True):

    # Create a datetime generator for generator for each process that iterates over each 5 minutes interval in the
    # start to end time window.
    delta = timedelta(minutes=5)
    start_time_obj = datetime.strptime(start_time, '%Y/%m/%d %H:%M:%S')
    end_time_obj = datetime.strptime(end_time, '%Y/%m/%d %H:%M:%S')

    date_times_generator_2 = datetime_dispatch_sequence(start_time_obj, end_time_obj, delta)
    if run_pre_filter:
        with Parallel(n_jobs=1) as pool:
            # Pre filter dispatch constraint first as some tables are filtered based of its filtered files.
            run_pf('DISPATCHCONSTRAINT', start_time, end_time, raw_aemo_data_folder, filtered_data_folder)
            # Pre filter the rest of the tables.
            pool(delayed(run_pf)(table, start_time, end_time, raw_aemo_data_folder, filtered_data_folder)
                 for table in nemlite_defaults.parent_tables if table != 'DISPATCHCONSTRAINT')

    for date_time in date_times_generator_2:
        datetime_name = date_time.replace('/', '')
        datetime_name = datetime_name.replace(" ", "_")
        datetime_name = datetime_name.replace(":", "")
        input_tables = load_and_merge(datetime_name, filtered_data_folder)

        yield input_tables['generator_information'], input_tables['capacity_bids'], input_tables['initial_conditions'],\
            input_tables['interconnectors'], input_tables['demand'], input_tables['price_bids'], \
            input_tables['interconnector_segments'], input_tables['connection_point_constraints'], \
            input_tables['interconnector_constraints'], input_tables['constraint_data'], \
            input_tables['region_constraints'], date_time, input_tables['interconnector_dynamic_loss_coefficients'],\
            input_tables['market_interconnectors'], input_tables['market_interconnector_price_bids'], \
            input_tables['market_interconnector_capacity_bids']


def run_pf(table, start_time, end_time, raw_aemo_data_folder, filtered_data_folder):
    start_time_obj = datetime.strptime(start_time, '%Y/%m/%d %H:%M:%S')
    end_time_obj = datetime.strptime(end_time, '%Y/%m/%d %H:%M:%S')
    delta = timedelta(minutes=5)
    date_times_generator_1 = datetime_dispatch_sequence(start_time_obj, end_time_obj, delta)
    pre_filter(start_time, end_time, table, date_times_generator_1, raw_aemo_data_folder, filtered_data_folder)


def load_and_merge(date_time_name, filtered_data):
    save_location_formated = filtered_data + '/{}_{}.csv'
    child_tables = {}
    for child_table_name, parent_tables_names in nemlite_defaults.child_parent_map.items():
        parent_tables = []
        for name in parent_tables_names:
            save_name_and_location = save_location_formated.format(name, date_time_name)
            parent_tables.append(pd.read_csv(save_name_and_location))
        if len(parent_tables) == 2:
            child_tables[child_table_name] = pd.merge(parent_tables[0], parent_tables[1], 'inner',
                                                      nemlite_defaults.parent_merge_cols[child_table_name])
        elif len(parent_tables) == 1:
            child_tables[child_table_name] = parent_tables[0]
        else:
            print('Parent table left unmerged')
    return child_tables


def pre_filter(start_time, end_time, table, date_time_sequence, raw_data_location, filtered_data):

    if 'INTERVENTION' in defaults.table_columns[table]:
        filter_cols = ['INTERVENTION']
        filter_values = [['0']]
    else:
        filter_cols = None
        filter_values = None

    all_data = data_fetch_methods.method_map[table](start_time, end_time, table, raw_data_location,
                                                    filter_cols=filter_cols, filter_values=filter_values)

    save_location_formated = filtered_data + '/{}_{}.csv'
    for date_time in date_time_sequence:
        datetime_name = date_time.replace('/', '')
        datetime_name = datetime_name.replace(" ", "_")
        datetime_name = datetime_name.replace(":", "")
        date_time_specific_data = filter_map[table](all_data, save_location_formated, date_time, datetime_name, table)
        if nemlite_defaults.required_cols[table] is not None:
            date_time_specific_data = date_time_specific_data.loc[:, nemlite_defaults.required_cols[table]]
        date_time_specific_data.to_csv(save_location_formated.format(table, datetime_name), sep=',', index=False,
                                       date_format='%Y/%m/%d %H:%M:%S')


def constraint_filter(constraint_data, save_location_formated, date_time, datetime_name, table_name):
    dispatch_cons_filename = save_location_formated.format('DISPATCHCONSTRAINT', datetime_name)
    dispatched_constraints = pd.read_csv(dispatch_cons_filename, dtype=str)
    merge_cols = ('GENCONID', 'EFFECTIVEDATE', 'VERSIONNO')
    dispatched_constraints = dispatched_constraints.loc[:, merge_cols]
    dispatched_constraints['EFFECTIVEDATE'] = pd.to_datetime(dispatched_constraints['EFFECTIVEDATE'])
    filtered_constraints = pd.merge(dispatched_constraints, constraint_data, 'inner',
                                    ['GENCONID', 'EFFECTIVEDATE', 'VERSIONNO'])
    filtered_constraints = filtered_constraints.drop_duplicates(defaults.table_primary_keys[table_name])

    return filtered_constraints


def settlement_date_filter(data, save_location_formated, date_time, datetime_name, table_name):
    date_time = datetime.strptime(date_time, '%Y/%m/%d %H:%M:%S')
    data['SETTLEMENTDATE'] = pd.to_datetime(data['SETTLEMENTDATE'], format='%Y/%m/%d %H:%M:%S')
    data = data[(data['SETTLEMENTDATE'] == date_time)]
    if table_name == 'DISPATCHCONSTRAINT':
        data = data.loc[:, ('CONSTRAINTID', 'RHS', 'GENCONID_VERSIONNO', 'GENCONID_EFFECTIVEDATE')]
        data.columns = ['GENCONID', 'RHS', 'VERSIONNO', 'EFFECTIVEDATE']

    return data


def interval_datetime_filter(data, save_location_formated, date_time, datetime_name, table_name):
    date_time = datetime.strptime(date_time, '%Y/%m/%d %H:%M:%S')
    data['INTERVAL_DATETIME'] = pd.to_datetime(data['INTERVAL_DATETIME'], format='%Y/%m/%d %H:%M:%S')
    data = data[(data['INTERVAL_DATETIME'] == date_time)]
    if table_name == 'DISPATCHCONSTRAINT':
        data = data.loc[:, nemlite_defaults.required_cols['DISPATCHCONSTRAINT']]
        data.columns = ['GENCONID', 'RHS', 'VERSIONNO', 'EFFECTIVEDATE']

    return data


def half_hour_peroids(data, save_location_formated, date_time, datetime_name, table_name):
    date_time = datetime.strptime(date_time, '%Y/%m/%d %H:%M:%S')
    data['SETTLEMENTDATE'] = pd.to_datetime(data['SETTLEMENTDATE'], format='%Y/%m/%d %H:%M:%S')
    data = data[(data['SETTLEMENTDATE'] >= date_time - timedelta(minutes=30) ) & (data['SETTLEMENTDATE'] < date_time)]
    group_cols = defaults.effective_date_group_col[table_name]
    data = most_recent_version(data, table_name, group_cols)
    return data


def settlement_just_date_filter(data, save_location_formated, date_time, datetime_name, table_name):
    date_time = datetime.strptime(date_time, '%Y/%m/%d %H:%M:%S')
    date_time = date_time - timedelta(hours=4, seconds=1)
    date = date_time.replace(hour=0, minute=0, second=0)
    data['SETTLEMENTDATE'] = pd.to_datetime(data['SETTLEMENTDATE'], format='%Y/%m/%d %H:%M:%S')
    data = data[(data['SETTLEMENTDATE'] == date)]
    return data


def settlement_just_date_and_version_filter(data, save_location_formated, date_time, datetime_name, table_name):
    date_time = datetime.strptime(date_time, '%Y/%m/%d %H:%M:%S')
    date_time = date_time - timedelta(hours=4, seconds=1)
    date = date_time.replace(hour=0, minute=0, second=0)
    data['SETTLEMENTDATE'] = pd.to_datetime(data['SETTLEMENTDATE'], format='%Y/%m/%d %H:%M:%S')
    data = data[(data['SETTLEMENTDATE'] == date)]
    group_cols = defaults.effective_date_group_col[table_name]
    data = most_recent_version(data, table_name, group_cols)
    return data


def effective_date_filter(data, save_location_formated, date_time, datetime_name, table_name):
    date_time = datetime.strptime(date_time, '%Y/%m/%d %H:%M:%S')
    data['EFFECTIVEDATE'] = pd.to_datetime(data['EFFECTIVEDATE'], format='%Y/%m/%d %H:%M:%S')
    data = data[data['EFFECTIVEDATE'] <= date_time]
    group_cols = defaults.effective_date_group_col[table_name]
    data = query_wrapers.most_recent_records_before_start_time(data, date_time, table_name)
    return data


def effective_date_and_version_filter(data, save_location_formated, date_time, datetime_name, table_name):
    date_time = datetime.strptime(date_time, '%Y/%m/%d %H:%M:%S')
    data['EFFECTIVEDATE'] = pd.to_datetime(data['EFFECTIVEDATE'], format='%Y/%m/%d %H:%M:%S')
    data = data[data['EFFECTIVEDATE'] <= date_time]
    group_cols = defaults.effective_date_group_col[table_name]
    data = query_wrapers.most_recent_records_before_start_time(data, date_time, table_name)
    data = most_recent_version(data, table_name, group_cols)
    return data


def effective_date_and_version_filter_for_inter_seg(data, save_location_formated, date_time, datetime_name, table_name):
    data_orginal = data.copy()
    date_time = datetime.strptime(date_time, '%Y/%m/%d %H:%M:%S')
    data['EFFECTIVEDATE'] = pd.to_datetime(data['EFFECTIVEDATE'], format='%Y/%m/%d %H:%M:%S')
    data = data[data['EFFECTIVEDATE'] <= date_time]
    group_cols = defaults.effective_date_group_col[table_name]
    data = query_wrapers.most_recent_records_before_start_time(data, date_time, table_name)
    data = most_recent_version(data, table_name, group_cols)
    data = data.drop_duplicates(['EFFECTIVEDATE', 'INTERCONNECTORID', 'VERSIONNO', 'LOSSSEGMENT'])
    #data = data.loc[:, ('EFFECTIVEDATE', 'INTERCONNECTORID', 'VERSIONNO')]
    #data = pd.merge(data, data_orginal, 'left', ['EFFECTIVEDATE', 'INTERCONNECTORID', 'VERSIONNO'])
    return data


def start_date_end_date_filter(data, save_location_formated, date_time, datetime_name, table_name):
    date_time = datetime.strptime(date_time, '%Y/%m/%d %H:%M:%S')
    data['START_DATE'] = pd.to_datetime(data['START_DATE'], format='%Y/%m/%d %H:%M:%S')
    data['END_DATE'] = pd.to_datetime(data['END_DATE'], format='%Y/%m/%d %H:%M:%S')
    data = data[(data['START_DATE'] <= date_time) & (data['END_DATE'] > date_time)]
    data = data.sort_values('END_DATE').groupby(['DUID', 'START_DATE'], as_index=False).first()
    data = query_wrapers.most_recent_records_before_start_time(data, date_time, table_name)
    return data


def no_filter(data, save_location_formated, date_time, datetime_name, table_name):
    return data


def most_recent_version(data, table_name, group_cols):
    data = data.sort_values('VERSIONNO')
    data_most_recent_v = data.groupby(group_cols, as_index=False).last()
    group_cols = group_cols + ['VERSIONNO']
    data_most_recent_v = pd.merge(data_most_recent_v.loc[:, group_cols], data, 'inner', group_cols)
    return data_most_recent_v


def datetime_dispatch_sequence(start_time, end_time, delta):
    # Generator to produce time stamps at set intervals. Requires a datetime object as an input, but outputs
    # the date time as string formatted as 'YYYY/MM/DD HH:MM:SS'.
    curr = start_time + delta
    while curr <= end_time:
        # Change the datetime object to a timestamp and modify it format by replacing characters.
        yield curr.isoformat().replace('T', ' ').replace('-', '/')
        curr += delta


def derive_group_cols(table_name, date_col, also_exclude=None):
    exclude_from_group_cols = [date_col, 'VERSIONNO']
    if also_exclude is not None:
        exclude_from_group_cols.append(also_exclude)
    group_cols = [column for column in defaults.table_primary_keys[table_name] if column not in exclude_from_group_cols]
    return group_cols

filter_map = {'SPDCONNECTIONPOINTCONSTRAINT': constraint_filter,
              'GENCONDATA': constraint_filter,
              'SPDINTERCONNECTORCONSTRAINT': constraint_filter,
              'BIDPEROFFER_D': interval_datetime_filter,
              'DISPATCHINTERCONNECTORRES': settlement_date_filter,
              'INTERCONNECTOR': no_filter,
              'INTERCONNECTORCONSTRAINT': effective_date_and_version_filter,
              'MNSP_INTERCONNECTOR': effective_date_and_version_filter,
              'DISPATCHPRICE': settlement_date_filter,
              'DUDETAILSUMMARY': start_date_end_date_filter,
              'DISPATCHCONSTRAINT': settlement_date_filter,
              'SPDREGIONCONSTRAINT': constraint_filter,
              'BIDDAYOFFER_D': settlement_just_date_filter,
              'MNSP_DAYOFFER': settlement_just_date_and_version_filter,
              'MNSP_PEROFFER': half_hour_peroids,
              'DISPATCHLOAD': settlement_date_filter,
              'LOSSMODEL': effective_date_and_version_filter_for_inter_seg,
              'LOSSFACTORMODEL': effective_date_and_version_filter,
              'DISPATCHREGIONSUM': settlement_date_filter}







import os
from nemlite import standards as st


def declare_names():
    aemo_naming = st.Naming(base_filename='/{}_{}{}010000.CSV',
                            col_datetime_applicable='INTERVAL_DATETIME',
                            col_date_applicable='SETTLEMENTDATE',
                            col_date_changed='LASTCHANGED',
                            cols_bid_cap_name_list=['BANDAVAIL1', 'BANDAVAIL2', 'BANDAVAIL3', 'BANDAVAIL4',
                                                    'BANDAVAIL5', 'BANDAVAIL6', 'BANDAVAIL7', 'BANDAVAIL8',
                                                    'BANDAVAIL9', 'BANDAVAIL10'],
                            cols_bid_price_name_list=['PRICEBAND1', 'PRICEBAND2', 'PRICEBAND3', 'PRICEBAND4',
                                                      'PRICEBAND5', 'PRICEBAND6', 'PRICEBAND7', 'PRICEBAND8',
                                                      'PRICEBAND9', 'PRICEBAND10'],
                            col_unit_name='DUID',
                            col_unit_max_output='MAXAVAIL',
                            col_enablement_min='ENABLEMENTMIN',
                            col_enablement_max='ENABLEMENTMAX',
                            col_bid_type='BIDTYPE',
                            col_capacity_band_number='CAPACITYBAND',
                            col_price_band_number='PRICEBAND',
                            col_bid_value='BID',
                            col_enablement_type='ENABLEMENTTYPE',
                            col_enablement_value='ENABLEMENTVALUE',
                            col_low_break_point='LOWBREAKPOINT',
                            col_high_break_point='HIGHBREAKPOINT',
                            col_lhs_coefficients='LHSCOEFFICIENTS',
                            col_rhs_constant='RHSCONSTANT',
                            col_upper_bound='UPPERBOUND',
                            col_lower_bound='LOWERBOUND',
                            col_variable_index='INDEX',
                            col_energy_variable_index='ENERGYINDEX',
                            col_max_unit_energy='MAXENERGY',
                            col_constraint_row_index='ROWINDEX',
                            col_fcas_integer_variable='FCASINTEGER',
                            type_energy='ENERGY',
                            type_fcas='FCAS',
                            col_inter_id='INTERCONNECTORID',
                            col_import_limit='IMPORTLIMIT',
                            col_export_limit='EXPORTLIMIT',
                            col_fcas_import_limit='FCASIMPORTLIMIT',
                            col_fcas_export_limit='FCASEXPORTLIMIT',
                            list_fcas_types=['LOWER5MIN', 'LOWER60SEC', 'LOWER6SEC', 'RAISE5MIN', 'RAISE60SEC',
                                             'RAISE6SEC', 'LOWERREG', 'RAISEREG'],
                            col_gen_req='TOTALDEMAND',
                            col_lower_5min_req='LOWER5MINLOCALDISPATCH',
                            col_lower_60sec_req='LOWER60SECLOCALDISPATCH',
                            col_lower_6sec_req='LOWER6SECLOCALDISPATCH',
                            col_raise_5min_req='RAISE5MINLOCALDISPATCH',
                            col_raise_60sec_req='RAISE60SECLOCALDISPATCH',
                            col_raise_6sec_req='RAISE6SECLOCALDISPATCH',
                            col_lower_reg_req='LOWERREGLOCALDISPATCH',
                            col_raise_req_req='RAISEREGLOCALDISPATCH',
                            col_region_id='REGIONID',
                            col_region_constraint_type='CONSTRAINTTYPE',
                            col_region_constraint_value='RHSCONSTANT',
                            col_end_date='END_DATE',
                            col_contribution_coefficients='CONTRACOE',
                            col_region_from='REGIONFROM',
                            col_region_to='REGIONTO',
                            col_direction='DIRECTION',
                            col_limit_value='LIMIT',
                            col_enquality_type='ENQUALITYTYPE',
                            col_price='PRICE',
                            col_loss_factor='TRANSMISSIONLOSSFACTOR',
                            col_dispatch_type='DISPATCHTYPE',
                            type_load='LOAD',
                            type_gen='GENERATOR',
                            col_pool_id='POOLID')
    return aemo_naming



class Naming:
    def __init__(self, base_filename, col_datetime_applicable: str, col_date_applicable: str,
                 col_date_changed: str, cols_bid_cap_name_list: list,
                 cols_bid_price_name_list: list, col_unit_name: str, col_unit_max_output: str, col_enablement_min: str,
                 col_enablement_max: str, col_bid_type: str, col_capacity_band_number: str, col_price_band_number: str,
                 col_bid_value: str,
                 col_enablement_type: str, col_enablement_value: str, col_low_break_point: str,
                 col_high_break_point: str, col_lhs_coefficients: str, col_rhs_constant: str, col_upper_bound: str,
                 col_lower_bound: str, col_variable_index: str, col_energy_variable_index: str,
                 col_max_unit_energy: str, col_constraint_row_index: str, col_fcas_integer_variable: str,
                 type_energy: str, col_inter_id: str, col_import_limit: str, col_export_limit: str,
                 col_fcas_import_limit: str, col_fcas_export_limit: str, list_fcas_types: list, type_fcas: str,
                 col_gen_req: str, col_lower_5min_req: str, col_lower_60sec_req: str, col_lower_6sec_req: str,
                 col_raise_5min_req: str, col_raise_60sec_req: str, col_raise_6sec_req: str, col_lower_reg_req: str,
                 col_raise_req_req: str, col_region_id: str, col_region_constraint_type: str,
                 col_region_constraint_value: str, col_end_date: str, col_contribution_coefficients: str,
                 col_region_from: str, col_region_to: str, col_direction: str, col_limit_value,
                 col_enquality_type: str, col_price: str, col_loss_factor: str, col_dispatch_type: str, type_load:str,
                 type_gen: str, col_pool_id: str):

        self.base_filename = base_filename
        self.col_datetime_applicable = col_datetime_applicable
        self.col_date_applicable = col_date_applicable
        self.col_date_changed = col_date_changed
        self.cols_bid_cap_name_list = cols_bid_cap_name_list
        self.cols_bid_price_name_list = cols_bid_price_name_list
        self.col_unit_name = col_unit_name
        self.col_unit_max_output = col_unit_max_output
        self.col_enablement_min = col_enablement_min
        self.col_enablement_max = col_enablement_max
        self.col_bid_type = col_bid_type
        self.col_capacity_band_number = col_capacity_band_number
        self.col_price_band_number = col_price_band_number
        self.col_bid_value = col_bid_value
        self.col_enablement_type = col_enablement_type
        self.col_enablement_value = col_enablement_value
        self.col_low_break_point = col_low_break_point
        self.col_high_break_point = col_high_break_point
        self.col_lhs_coefficients = col_lhs_coefficients
        self.col_rhs_constant = col_rhs_constant
        self.col_upper_bound = col_upper_bound
        self.col_lower_bound = col_lower_bound
        self.col_variable_index = col_variable_index
        self.col_energy_variable_index = col_energy_variable_index
        self.col_max_unit_energy = col_max_unit_energy
        self.col_constraint_row_index = col_constraint_row_index
        self.col_fcas_integer_variable = col_fcas_integer_variable
        self.type_energy = type_energy
        self.type_fcas = type_fcas
        self.col_inter_id = col_inter_id
        self.col_import_limit = col_import_limit
        self.col_export_limit = col_export_limit
        self.col_fcas_import_limit = col_fcas_import_limit
        self.col_fcas_export_limit = col_fcas_export_limit
        self.list_region_req_cols = [col_gen_req, col_lower_5min_req, col_lower_60sec_req, col_lower_6sec_req,
                                     col_raise_5min_req, col_raise_60sec_req, col_raise_6sec_req, col_lower_reg_req,
                                     col_raise_req_req]
        self.list_fcas_types = list_fcas_types
        self.col_region_id = col_region_id
        self.col_region_constraint_type = col_region_constraint_type
        self.col_region_constraint_value = col_region_constraint_value
        self.req_suffix = 'LOCALDISPATCH'
        self.col_end_date = col_end_date
        self.col_contribution_coefficients = col_contribution_coefficients
        self.col_region_from = col_region_from
        self.col_region_to = col_region_to
        self.col_direction = col_direction
        self.col_gen_req = col_gen_req
        self.col_limit_value = col_limit_value
        self.col_enquality_type = col_enquality_type
        self.col_price = col_price
        self.col_loss_factor = col_loss_factor
        self.col_dispatch_type = col_dispatch_type
        self.type_load = type_load
        self.type_gen = type_gen
        self.col_pool_id = col_pool_id
